@@ -73,7 +73,7 @@ MathStructure Calculator::convertToMixedUnits(const MathStructure &mstruct, cons
 			if(u->subtype() == SUBTYPE_ALIAS_UNIT && ((AliasUnit*) u)->firstBaseExponent() == 1 && ((AliasUnit*) u)->firstBaseUnit()->subtype() != SUBTYPE_COMPOSITE_UNIT) {
 				size_t idiv = ((AliasUnit*) u)->expression().find("1" DIVISION);
 				if(idiv == 0 && ((AliasUnit*) u)->expression().find_first_not_of(NUMBERS, 2) == string::npos) {
-					if((((AliasUnit*) u)->mixWithBase() > 0 || (((AliasUnit*) u)->mixWithBase() == 0 && (muc == MIXED_UNITS_CONVERSION_FORCE_INTEGER || muc == MIXED_UNITS_CONVERSION_FORCE_ALL))) && (((AliasUnit*) u)->mixWithBaseMinimum() <= 1 || nr.isGreaterThanOrEqualTo(((AliasUnit*) u)->mixWithBaseMinimum()))) {
+					if((((AliasUnit*) u)->mixWithBase() > 0 || (((AliasUnit*) u)->mixWithBase() <= 0 && (muc == MIXED_UNITS_CONVERSION_FORCE_INTEGER || muc == MIXED_UNITS_CONVERSION_FORCE_ALL))) && (((AliasUnit*) u)->mixWithBaseMinimum() <= 1 || nr.isGreaterThanOrEqualTo(((AliasUnit*) u)->mixWithBaseMinimum()))) {
 						best_u = ((AliasUnit*) u)->firstBaseUnit();
 						MathStructure mstruct_nr(nr);
 						MathStructure m_exp(m_one);
@@ -139,7 +139,7 @@ MathStructure Calculator::convertToMixedUnits(const MathStructure &mstruct, cons
 				mstruct_nr.eval(eo);
 				while(!accept_obsolete && ((AliasUnit*) u)->firstBaseUnit()->subtype() == SUBTYPE_ALIAS_UNIT && abs(((AliasUnit*) ((AliasUnit*) u)->firstBaseUnit())->mixWithBase()) > 1) {
 					u = ((AliasUnit*) u)->firstBaseUnit();
-					if(((AliasUnit*) u)->firstBaseExponent() == 1 && (((AliasUnit*) u)->mixWithBase() != 0 || muc == MIXED_UNITS_CONVERSION_FORCE_ALL || (muc == MIXED_UNITS_CONVERSION_FORCE_INTEGER && ((AliasUnit*) u)->expression().find_first_not_of(NUMBERS) == string::npos))) {
+					if(((AliasUnit*) u)->firstBaseExponent() == 1 && (((AliasUnit*) u)->mixWithBase() > 0 || muc == MIXED_UNITS_CONVERSION_FORCE_ALL || (muc == MIXED_UNITS_CONVERSION_FORCE_INTEGER && ((AliasUnit*) u)->expression().find_first_not_of(NUMBERS) == string::npos))) {
 						((AliasUnit*) u)->convertToFirstBaseUnit(mstruct_nr, m_exp);
 						mstruct_nr.eval(eo);
 						if(!mstruct_nr.isNumber() || !m_exp.isOne()) break;
@@ -156,7 +156,7 @@ MathStructure Calculator::convertToMixedUnits(const MathStructure &mstruct, cons
 							non_int = true;
 						}
 					} else {
-						if(((AliasUnit*) u)->mixWithBase() != 0 && muc > MIXED_UNITS_CONVERSION_DEFAULT) muc = MIXED_UNITS_CONVERSION_DEFAULT;
+						if(((AliasUnit*) u)->mixWithBase() > 0 && muc > MIXED_UNITS_CONVERSION_DEFAULT) muc = MIXED_UNITS_CONVERSION_DEFAULT;
 						u = ((AliasUnit*) u)->firstBaseUnit();
 						nr = mstruct_nr.number();
 						b = true;
@@ -888,15 +888,28 @@ MathStructure Calculator::convert(const MathStructure &mstruct, Unit *to_unit, c
 			fix_to_struct(munit);
 			if(!munit.isZero()) {
 				MathStructure mstruct_new(mstruct);
-				mstruct_new *= munit;
-				PrintOptions po = message_printoptions;
-				po.negative_exponents = false;
-				munit.format(po);
-				if(munit.isMultiplication() && munit.size() >= 2) {
-					if(munit[0].isOne()) munit.delChild(1, true);
-					else if(munit[1].isOne()) munit.delChild(2, true);
+				if(mstruct_new.isComparison() || (mstruct_new.isLogicalAnd() && mstruct_new.size() > 0 && mstruct_new[0].isComparison()) || (mstruct_new.isLogicalOr() && mstruct_new.size() > 0 && (mstruct_new[0].isComparison() || (mstruct_new[0].isLogicalAnd() && mstruct_new[0].size() > 0 && mstruct_new[0][0].isComparison())))) {
+					if(mstruct_new.isLogicalOr()) {
+						for(size_t i = 0; i < mstruct_new.size(); i++) {
+							if(mstruct_new[i].isComparison()) mstruct_new[i][1] *= munit;
+							else if(mstruct_new[i].isLogicalAnd() && mstruct_new[i].size() > 0 && mstruct_new[i][0].isComparison()) mstruct_new[i][0][1] *= munit;
+						}
+					} else {
+						if(mstruct_new.isLogicalAnd()) mstruct_new[0][1] *= munit;
+						else mstruct_new[1] *= munit;
+					}
+					mstruct_new.childrenUpdated(true);
+				} else {
+					mstruct_new *= munit;
+					PrintOptions po = message_printoptions;
+					po.negative_exponents = false;
+					munit.format(po);
+					if(munit.isMultiplication() && munit.size() >= 2) {
+						if(munit[0].isOne()) munit.delChild(1, true);
+						else if(munit[1].isOne()) munit.delChild(2, true);
+					}
+					if(parsed_struct) parsed_struct->multiply(munit, true);
 				}
-				if(parsed_struct) parsed_struct->multiply(munit, true);
 				return convert(mstruct_new, to_unit, eo, always_convert, convert_to_mixed_units, false, NULL);
 			}
 		}
@@ -905,6 +918,19 @@ MathStructure Calculator::convert(const MathStructure &mstruct, Unit *to_unit, c
 	MathStructure mstruct_new(mstruct);
 	size_t n_messages = messages.size();
 	if(to_unit->hasNonlinearRelationTo(to_unit->baseUnit()) && to_unit->baseUnit()->subtype() == SUBTYPE_COMPOSITE_UNIT) {
+		if(mstruct_new.isComparison() || (mstruct_new.isLogicalAnd() && mstruct_new.size() > 0 && mstruct_new[0].isComparison()) || (mstruct_new.isLogicalOr() && mstruct_new.size() > 0 && (mstruct_new[0].isComparison() || (mstruct_new[0].isLogicalAnd() && mstruct_new[0].size() > 0 && mstruct_new[0][0].isComparison())))) {
+			if(mstruct_new.isLogicalOr()) {
+				for(size_t i = 0; i < mstruct_new.size(); i++) {
+					if(mstruct_new[i].isComparison()) mstruct_new[i][1].set(convert(mstruct_new[i][1], to_unit, eo, always_convert, convert_to_mixed_units, transform_orig));
+					else if(mstruct_new[i].isLogicalAnd() && mstruct_new[i].size() > 0 && mstruct_new[i][0].isComparison()) mstruct_new[i][0][1].set(convert(mstruct_new[i][0][1], to_unit, eo, always_convert, convert_to_mixed_units, transform_orig));
+				}
+			} else {
+				if(mstruct_new.isLogicalAnd()) mstruct_new[0][1].set(convert(mstruct_new[0][1], to_unit, eo, always_convert, convert_to_mixed_units, transform_orig));
+				else mstruct_new[1].set(convert(mstruct_new[1], to_unit, eo, always_convert, convert_to_mixed_units, transform_orig));
+			}
+			mstruct_new.childrenUpdated(true);
+			return mstruct_new;
+		}
 		mstruct_new = convert(mstruct, to_unit->baseUnit(), eo, always_convert, convert_to_mixed_units, transform_orig, parsed_struct);
 		mstruct_new.calculateDivide(((CompositeUnit*) to_unit->baseUnit())->generateMathStructure(false, eo.keep_prefixes), eo);
 		to_unit->convertFromBaseUnit(mstruct_new);
@@ -930,7 +956,7 @@ MathStructure Calculator::convert(const MathStructure &mstruct, Unit *to_unit, c
 		if(mstruct_new.size() > 0) {
 			for(size_t i = 0; i < mstruct_new.size(); i++) {
 				if(aborted()) return mstruct;
-				if(!mstruct_new.isFunction() || !mstruct_new.function()->getArgumentDefinition(i + 1) || mstruct_new.function()->getArgumentDefinition(i + 1)->type() == ARGUMENT_TYPE_FREE || (!mstruct_new.function()->getArgumentDefinition(i + 1)->tests() && mstruct_new.function()->getArgumentDefinition(i + 1)->type() != ARGUMENT_TYPE_ANGLE)) {
+				if((i != 0 || !mstruct_new.isComparison() || mstruct[0].containsType(STRUCT_UNIT) > 0) && (!mstruct_new.isFunction() || !mstruct_new.function()->getArgumentDefinition(i + 1) || mstruct_new.function()->getArgumentDefinition(i + 1)->type() == ARGUMENT_TYPE_FREE || (!mstruct_new.function()->getArgumentDefinition(i + 1)->tests() && mstruct_new.function()->getArgumentDefinition(i + 1)->type() != ARGUMENT_TYPE_ANGLE))) {
 					mstruct_new[i] = convert(mstruct_new[i], to_unit, eo, false, convert_to_mixed_units, false, NULL);
 					if(!b_changed && !mstruct_new.equals(mstruct[i], true, true)) b_changed = true;
 				}
@@ -1192,16 +1218,20 @@ MathStructure Calculator::convert(const MathStructure &mstruct, Unit *to_unit, c
 				mstruct_new.eval(eo2);
 				eo2.mixed_units_conversion = MIXED_UNITS_CONVERSION_NONE;
 				bool b_pos = !cu, b_neg = false;
-				if(cu || to_unit->baseUnit()->subtype() == SUBTYPE_COMPOSITE_UNIT) {
-					CompositeUnit *cu2 = cu;
-					if(cu2) b_pos = true;
-					else cu2 = (CompositeUnit*) to_unit->baseUnit();
+				CompositeUnit *cu2 = cu;
+				if(cu2) b_pos = true;
+				else if(to_unit->baseUnit()->subtype() == SUBTYPE_COMPOSITE_UNIT) cu2 = (CompositeUnit*) to_unit->baseUnit();
+				if(cu2) {
 					int exp2 = 1;
 					for(size_t i = 1; i <= cu2->countUnits(); i++) {
 						cu2->get(i, &exp2);
+						if(!cu) exp2 *= to_unit->baseExponent();
+						exp2 *= cu2->get(i)->baseExponent();
 						if(exp2 < 0) b_neg = true;
 						else b_pos = true;
 					}
+				} else {
+					b_neg = to_unit->baseExponent() < 0;
 				}
 				long int n = count_unit_powers(mstruct_new);
 				test_convert(mstruct_new, to_unit, n, b_pos, eo2);
