@@ -871,8 +871,8 @@ bool calculate_userfunctions(MathStructure &m, const MathStructure &x_mstruct, c
 			b_ret = true;
 		}
 	}
-	if(m.isFunction()) {
-		if(!m.contains(x_mstruct, true) && !m.containsFunctionId(FUNCTION_ID_RAND, true, true, true) && !m.containsFunctionId(FUNCTION_ID_RANDN, true, true, true) && !m.containsFunctionId(FUNCTION_ID_RAND_POISSON, true, true, true)) {
+	if(m.isFunction() && !contains_rand(m, true)) {
+		if(!m.contains(x_mstruct, true)) {
 			if(m.calculateFunctions(eo, false)) {
 				b_ret = true;
 				calculate_userfunctions(m, x_mstruct, eo, b_vector, depth + 1);
